@@ -10,6 +10,8 @@ use thiserror::Error;
 
 use crate::{
     command::CommandType,
+    elf::{FlashFrequency, FlashMode},
+    flasher::FlashSize,
     image_format::ImageFormatId,
     partition_table::{SubType, Type},
     Chip,
@@ -75,14 +77,40 @@ https://github.com/espressif/esp32c3-direct-boot-example"
         )
     )]
     InvalidDirectBootBinary,
-    #[error("No serial port specified in arguments or config")]
+    #[error("No serial ports could be detected")]
     #[diagnostic(
-        code(cargo_espflash::no_serial),
-        help("Add a command line option with the serial port to use")
+        code(espflash::no_serial),
+        help("Make sure you have connected a device to the host system")
     )]
     NoSerial,
+    #[error("The serial port '{0}' could not be found")]
+    #[diagnostic(
+        code(espflash::serial_not_found),
+        help("Make sure the correct device is connected to the host system")
+    )]
+    SerialNotFound(String),
     #[error("Canceled by user")]
     Canceled,
+    #[error("The flash mode '{0}' is not valid")]
+    #[diagnostic(
+        code(espflash::invalid_flash_mode),
+        help("The accepted values are: {:?}", FlashMode::VARIANTS)
+    )]
+    InvalidFlashMode(String),
+    #[error("The flash frequency '{0}' is not valid")]
+    #[diagnostic(
+        code(espflash::invalid_flash_frequency),
+        help("The accepted values are: {:?}", FlashFrequency::VARIANTS)
+    )]
+    InvalidFlashFrequency(String),
+    #[error("The flash size '{0}' is not valid")]
+    #[diagnostic(
+        code(espflash::invalid_flash_size),
+        help("The accepted values are: {:?}", FlashSize::VARIANTS)
+    )]
+    InvalidFlashSize(String),
+    #[error("The provided bootloader binary is not valid")]
+    InvalidBootloader,
 }
 
 #[derive(Error, Debug, Diagnostic)]
@@ -383,7 +411,8 @@ impl CSVError {
     }
 }
 
-/// since csv doesn't give us the position in the line the error occurs, we highlight the entire line
+/// since csv doesn't give us the position in the line the error occurs, we
+/// highlight the entire line
 ///
 /// line starts at 1
 fn line_to_span(source: &str, line: usize) -> SourceSpan {
